'use strict';

var _mupApi = require('./mup-api');

var _mupApi2 = _interopRequireDefault(_mupApi);

var _updates = require('./updates');

var _updates2 = _interopRequireDefault(_updates);

var _modules = require('./modules/');

var _modules2 = _interopRequireDefault(_modules);

var _package = require('../package.json');

var _package2 = _interopRequireDefault(_package);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var settingsPath = void 0;
var configPath = void 0;
var args = process.argv.slice(2);

_commander2.default.arguments('<command> [subcommand]').version(_package2.default.version).action(argAction).option('--settings <filePath>', 'Meteor settings file', setSettingsPath).option('--config <filePath>', 'mup.js config file', setConfigPath).on('--help', function () {
  console.log('   Commands:');

  function listModuleCommands(commands) {
    Object.keys(commands).forEach(function (command) {
      if (command === 'default') {
        listModuleCommands(commands['default']);
        return;
      }
      console.log('     ' + command);
    });
  }

  listModuleCommands(_modules2.default);

  console.log('');
  console.log('    For list of subcommands, run ');
  console.log('      mup <command> help');
}).parse(process.argv);

if (_commander2.default.args.length === 0) {
  _commander2.default.help();
  process.exit(0);
}

function argAction(arg, subarg) {
  var moduleArg = arg;
  var command = subarg;

  if (!command && !_modules2.default[moduleArg]) {
    command = moduleArg;
    moduleArg = 'default';
  }

  if (moduleArg === 'default' && command === 'help') {
    _commander2.default.help();
    process.exit();
  }

  var module = void 0;

  if (_modules2.default[moduleArg]) {
    module = _modules2.default[moduleArg];
  } else {
    console.error('No such module ' + moduleArg);
    _commander2.default.help();
    process.exit(1);
  }

  if (!command) {
    if (moduleArg === 'default') {
      _commander2.default.help();
    } else {
      module.help(args);
    }
    process.exit(0);
  }

  if (!module[command]) {
    console.error('error: unknown command %s', command);
    if (moduleArg === 'default') {
      _commander2.default.help();
      process.exit(1);
    }

    module.help(args);
    process.exit(1);
  }

  if (_commander2.default.settings) {
    var settingsIndex = argIndex(args, '--settings');

    if (args[settingsIndex].indexOf('--settings=') === 0) {
      args.splice(settingsIndex, 1);
    } else {
      args.splice(settingsIndex, 2);
    }
  }

  if (_commander2.default.config) {
    var configIndex = argIndex(args, '--config');

    if (args[configIndex].indexOf('--config=') === 0) {
      args.splice(configIndex, 1);
    } else {
      args.splice(configIndex, 2);
    }
  }

  (0, _updates2.default)().then(function () {
    var base = process.cwd();
    var api = new _mupApi2.default(base, args, configPath, settingsPath);
    module[command](api);
  });
}

function argIndex(list, string) {
  for (var i = 0; i < list.length; i++) {
    if (list[i].indexOf(string) === 0) {
      return i;
    }
  }
}

function handleErrors(e) {
  console.log(e.name, e.message);
  process.exit(1);
}

function setSettingsPath(settingsPathArg) {
  settingsPath = settingsPathArg;
}

function setConfigPath(configPathArg) {
  configPath = configPathArg;
}

process.on('uncaughtException', handleErrors);