'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = validate;

var _utils = require('./utils');

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

var _meteor = require('./meteor');

var _meteor2 = _interopRequireDefault(_meteor);

var _mongo = require('./mongo');

var _mongo2 = _interopRequireDefault(_mongo);

var _servers = require('./servers');

var _servers2 = _interopRequireDefault(_servers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var schema = _joi2.default.object().keys({
  servers: _joi2.default.object(),
  meteor: _joi2.default.object(),
  mongo: _joi2.default.object()
});

function validateAll(config) {
  var details = [];
  var results = void 0;

  results = _joi2.default.validate(config, schema);
  details = (0, _utils.combineErrorDetails)(details, results);

  if (config.servers) {
    results = (0, _servers2.default)(config.servers);
    details = (0, _utils.combineErrorDetails)(details, results);
  }

  if (config.mongo) {
    results = (0, _mongo2.default)(config);
    details = (0, _utils.combineErrorDetails)(details, results);
  }

  if (config.meteor) {
    results = (0, _meteor2.default)(config);
    details = (0, _utils.combineErrorDetails)(details, results);
  }

  return details.map(_utils.improveErrors);
}

function validate(config) {
  var errors = validateAll(config);
  return {
    errors: errors.reduce(function (result, error) {
      if (!error.warning) {
        result.push(error.message);
      }
      return result;
    }, []),
    warnings: errors.reduce(function (result, error) {
      if (error.warning) {
        result.push(error.message);
      }
      return result;
    }, []),
    hasErrors: errors.reduce(function (result, error) {
      if (error.warning !== true) {
        return true;
      }
      return false;
    }, false)
  };
}