'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (config) {
  var details = [];
  details = (0, _utils.combineErrorDetails)(details, _joi2.default.validate(config.meteor, schema, { convert: false }));
  if (config.meteor.name.indexOf(' ') > -1) {
    details.push({
      message: '"name" has a space'
    });
  }
  details = (0, _utils.combineErrorDetails)(details, (0, _utils.serversExist)(config.servers, config.meteor.servers));

  return (0, _utils.addLocation)(details, 'meteor');
};

var _utils = require('./utils');

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var schema = _joi2.default.object().keys({
  name: _joi2.default.string().min(1).required(),
  path: _joi2.default.string().min(1).required(),
  port: _joi2.default.number(),
  servers: _joi2.default.object().required(),
  deployCheckWaitTime: _joi2.default.number(),
  enableUploadProgressBar: _joi2.default.bool(),
  dockerImage: _joi2.default.string(),
  setupVirtualHost: _joi2.default.bool(),
  mongoName: _joi2.default.string(),
  docker: _joi2.default.object().keys({
    image: _joi2.default.string().trim(),
    imageFrontendServer: _joi2.default.string(),
    args: _joi2.default.array().items(_joi2.default.string().label('docker.args array items'))
  }),
  buildOptions: _joi2.default.object().keys({
    serverOnly: _joi2.default.bool(),
    debug: _joi2.default.bool(),
    cleanAfterBuild: _joi2.default.bool(),
    mobileSettings: _joi2.default.object(),
    server: _joi2.default.string().uri(),
    allowIncompatibleUpdates: _joi2.default.boolean(),
    executable: _joi2.default.string()
  }),
  env: _joi2.default.object().keys({
    ROOT_URL: _joi2.default.string().regex(new RegExp('^(http|https)://', 'i')).required(),
    MONGO_URL: _joi2.default.string()
  }).pattern(/[\s\S]*/, [_joi2.default.string(), _joi2.default.number()]),
  log: _joi2.default.object().keys({
    driver: _joi2.default.string(),
    opts: _joi2.default.object()
  }),
  volumes: _joi2.default.object(),
  nginx: _joi2.default.object().keys({
    clientUploadLimit: _joi2.default.string().trim()
  }),
  ssl: _joi2.default.object().keys({
    autogenerate: _joi2.default.object().keys({
      email: _joi2.default.string().email().required(),
      domains: _joi2.default.string().required()
    }).label('autogenerate'),
    crt: _joi2.default.string().trim(),
    key: _joi2.default.string().trim(),
    port: _joi2.default.number()
  }).and('crt', 'key').without('autogenerate', ['crt', 'key']).or('crt', 'autogenerate').label('ssl')
});